@@ -31,7 +31,7 @@ int ppp_link_dispose();
 int ppp_link_control(struct ppp_link *link, u_int32_t cmd, void *data);
 int ppp_link_attachclient(u_short index, void *host, struct ppp_link **link);
 int ppp_link_detachclient(struct ppp_link *link, void *host);
-int ppp_link_send(struct ppp_link *link, struct mbuf *m);
+int ppp_link_send(struct ppp_link *link, mbuf_t m);
 
 
 #endif /* _PPP_LINK_H_ */
