@@ -23,23 +23,46 @@
 /*
  * options.c - handles option processing for PPP.
  *
- * Copyright (c) 1989 Carnegie Mellon University.
- * All rights reserved.
+ * Copyright (c) 1984-2000 Carnegie Mellon University. All rights reserved.
  *
- * Redistribution and use in source and binary forms are permitted
- * provided that the above copyright notice and this paragraph are
- * duplicated in all such forms and that any documentation,
- * advertising materials, and other materials related to such
- * distribution and use acknowledge that the software was developed
- * by Carnegie Mellon University.  The name of the
- * University may not be used to endorse or promote products derived
- * from this software without specific prior written permission.
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any legal
+ *    details, please contact
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#define RCSID	"$Id: options.c,v 1.10.38.1 2004/02/12 22:00:45 callie Exp $"
+#define RCSID	"$Id: options.c,v 1.20 2005/02/27 21:10:15 lindak Exp $"
 
 #include <ctype.h>
 #include <stdio.h>
@@ -52,6 +75,9 @@
 #include <pwd.h>
 #ifdef PLUGIN
 #ifdef __APPLE__
+#include <sys/un.h>
+#include <sys/ucred.h>
+#include <sys/socket.h>
 #else
 #include <dlfcn.h>
 #endif
@@ -68,7 +94,9 @@
 char *strdup __P((char *));
 #endif
 
+#ifndef lint
 static const char rcsid[] = RCSID;
+#endif
 
 struct option_value {
     struct option_value *next;
@@ -91,8 +119,13 @@ bool	updetach = 0;		/* Detach once link is up */
 int	maxconnect = 0;		/* Maximum connect time */
 char	user[MAXNAMELEN];	/* Username for PAP */
 #ifdef __APPLE__
-int 	optionsfd = -1;		/* parameters are given via pipe */
-char	username[MAXNAMELEN];	/* copy oroginal user */
+bool	controlled = 0;		/* Is pppd controlled by the PPPController ?  */
+FILE 	*controlfile = NULL;	/* file descriptor for options and control */
+int 	controlfd = -1;		/* file descriptor for options and control */
+uid_t 	controlfd_uid = -1;	/* uid at the other end of the control file descriptor*/
+int 	statusfd = -1;		/* file descriptor status update */
+char	username[MAXNAMELEN];	/* copy original user */
+char	new_passwd[MAXSECRETLEN];	/* new password for protocol supporting changing password */
 #endif
 char	passwd[MAXSECRETLEN];	/* Password for PAP */
 bool	persist = 0;		/* Reopen link after it goes down */
@@ -101,6 +134,7 @@ bool	demand = 0;		/* do dial-on-demand */
 char	*ipparam = NULL;	/* Extra parameter for ip up/down scripts */
 int	idle_time_limit = 0;	/* Disconnect if idle for this many seconds */
 bool   	noidlerecv = 0;         /* Disconnect if idle only for outgoing traffic */
+bool   	noidlesend = 0;         /* Disconnect if idle only for incoming traffic */
 int	holdoff = 30;		/* # seconds to pause before reconnecting */
 bool	holdoff_specified;	/* true if a holdoff value has been given */
 int	log_to_fd = 1;		/* send log messages to this fd too */
@@ -130,7 +164,10 @@ int 	redialcount = 0;	/* number of time to redial */
 int 	redialtimer = 30;	/* delay in seconds to wait before to redial */
 bool 	redialalternate = 0;  	/* do we redial alternate number */
 int 	busycode = -1;		/* busy error code that triggers the redial */
+bool	hasbusystate = 1;	/* change phase to report busy state */
 int 	cancelcode = -1;	/* cancel error code for connectors */
+int 	extraconnecttime = 0;	/* allows extra connection time to the connection sequence */
+bool	holdfirst = 0;	/* apply holdoff timer when starting pppd, useful to delay dialondemand */
 #endif
 
 
@@ -165,6 +202,9 @@ static int setlogfile __P((char **));
 #ifdef PLUGIN
 static int loadplugin __P((char **));
 #endif
+#ifdef __APPLE__
+static int controlled_connection __P((char **));
+#endif
 
 #ifdef PPP_FILTER
 static int setpassfilter __P((char **));
@@ -215,9 +255,15 @@ option_t general_options[] = {
       "Set time in seconds before retrying connection", OPT_PRIO },
 
     { "idle", o_int, &idle_time_limit,
-      "Set time in seconds before disconnecting idle link", OPT_PRIO },
+      "Set time in seconds before disconnecting idle link", OPT_PRIO, 0, 0, 0xFFFFFFFF, 0, 0, 0, option_change_idle },
+#ifdef __APPLE__
+    { "holdfirst", o_bool, &holdfirst,
+      "Apply holdoff timer first", OPT_PRIO | 1 },
     { "noidlerecv", o_bool, &noidlerecv,
       "Don't check receive traffic for idle timer", OPT_PRIO | 1 },
+    { "noidlesend", o_bool, &noidlesend,
+      "Don't check send traffic for idle timer", OPT_PRIO | 1 },
+#endif
 
     { "maxconnect", o_int, &maxconnect,
       "Set connection time limit",
@@ -330,8 +376,8 @@ option_t general_options[] = {
 #endif
 
 #ifdef __APPLE__
-    { "optionsfd", o_int, &optionsfd,
-      "File descriptor for parameters via pipe" },
+    { "controlled", o_special_noarg, (void *)controlled_connection,
+      "pppd is controlled by PPPController"},
     { "device", o_string, &device,
       "Device we are using"},
     { "remoteaddress", o_string, &remoteaddress,
@@ -348,6 +394,8 @@ option_t general_options[] = {
       "Busy signal error code that will trigger the redial"},
     { "cancelcode", o_int, &cancelcode,
       "Cancel error code that for connectors"},
+    { "extraconnecttime", o_int, &extraconnecttime,
+      "Allows extra conneciton time to the connection sequence"},
 #endif
 
     { NULL }
@@ -481,6 +529,10 @@ options_from_file(filename, must_exist, check_prot, priv)
 	}
 	if (!process_option(opt, cmd, argv))
 	    goto err;
+#ifdef __APPLE__        
+        // option source is saved by process option, don't free it 
+        tofree = 0;
+#endif
     }
     ret = 1;
 
@@ -495,67 +547,6 @@ err:
     return ret;
 }
 
-/*
- * options_from_file - Read a string of options from a file descriptor,
- * and interpret them.
- */
-int
-options_from_fd(fd)
-    int fd;
-{
-    FILE *f;
-    int i, fd1, newline, ret, err;
-    option_t *opt;
-    int n;
-    char *argv[MAXARGS];
-    char args[MAXARGS][MAXWORDLEN];
-    char cmd[MAXWORDLEN];
-
-    fd1 = dup(fd);
-    if (fd1 == -1) {
-	option_error("Can't duplicate options file descripor %d: %m", fd);
-	return 0;
-    }
-    
-    f = fdopen(fd1, "r");
-    if (f == NULL) {
-        close(fd1);
-	option_error("Can't open options file from file descripor %d: %m", fd);
-	return 0;
-    }
-
-    privileged_option = privileged;
-    option_source = "command line";
-    option_priority = OPRIO_CMDLINE;
-    ret = 0;
-
-    while (getword(f, cmd, &newline, "file descriptor")) {
-	opt = find_option(cmd);
-	if (opt == NULL) {
-	    option_error("In file descriptor %d: unrecognized option '%s'",
-			 fd, cmd);
-	    goto err;
-	}
-	n = n_arguments(opt);
-	for (i = 0; i < n; ++i) {
-	    if (!getword(f, args[i], &newline, "file descriptor")) {
-		option_error(
-			"In file descriptor %d: too few parameters for option '%s'",
-			fd, cmd);
-		goto err;
-	    }
-	    argv[i] = args[i];
-	}
-	if (!process_option(opt, cmd, argv))
-	    goto err;
-    }
-    ret = 1;
-
-err:
-    fclose(f); // also closes fd
-    return ret;
-}
-
 /*
  * options_from_user - See if the use has a ~/.ppprc file,
  * and if so, interpret options from it.
@@ -599,8 +590,8 @@ options_for_tty()
     size_t pl;
 
     dev = devnam;
-    if (strncmp(dev, "/dev/", 5) == 0)
-	dev += 5;
+    if ((p = strstr(dev, "/dev/")) != NULL)
+	dev = p + 5;
     if (dev[0] == 0 || strcmp(dev, "tty") == 0)
 	return 1;		/* don't look for /etc/ppp/options.tty */
     pl = strlen(_PATH_TTYOPT) + strlen(dev) + 1;
@@ -704,9 +695,9 @@ find_option(name)
 			if (match_option(name, opt, dowild))
 				return opt;
 		for (list = extra_options; list != NULL; list = list->next)
-			for (opt = list->options; opt->name != NULL; ++opt){
+			for (opt = list->options; opt->name != NULL; ++opt)
 				if (match_option(name, opt, dowild))
-					return opt;}
+					return opt;
 		for (opt = the_channel->options; opt->name != NULL; ++opt)
 			if (match_option(name, opt, dowild))
 				return opt;
@@ -732,6 +723,7 @@ process_option(opt, cmd, argv)
     int iv, a;
 #ifdef __APPLE__
     int i, len;
+    void (*change) __P((void));
  #endif
     char *sv;
     int (*parser) __P((char **));
@@ -740,8 +732,7 @@ process_option(opt, cmd, argv)
     int prio = option_priority;
     option_t *mainopt = opt;
 
-    current_option = cmd;
-
+    current_option = opt->name;
     if ((opt->flags & OPT_PRIVFIX) && privileged_option)
 	prio += OPRIO_ROOT;
     while (mainopt->flags & OPT_PRIOSUB)
@@ -908,7 +899,14 @@ process_option(opt, cmd, argv)
 #ifdef __APPLE__
     if (opt->type == o_string && opt->flags & OPT_HIDE)
         for (i = 0, len = strlen(*argv); i < len; (*argv)[i++] = '*');
+        
+    // call the change function
+    if (phase != PHASE_INITIALIZE && opt->addr3) {
+	change = (void (*) __P((void))) opt->addr3;
+	(*change)();
+    }
  #endif
+
     return 1;
 }
 
@@ -1153,7 +1151,11 @@ showversion(argv)
     char **argv;
 {
     if (phase == PHASE_INITIALIZE) {
+#ifdef __APPLE__
+	fprintf(stderr, "pppd version %s (Apple version %s)\n", VERSION, PPP_VERSION);
+#else
 	fprintf(stderr, "pppd version %s\n", VERSION);
+#endif
 	exit(0);
     }
     return 0;
@@ -1611,7 +1613,6 @@ setdomain(argv)
     return (1);
 }
 
-
 static int
 setlogfile(argv)
     char **argv;
@@ -1644,6 +1645,7 @@ setlogfile(argv)
     log_default = 0;
     return 1;
 }
+
 #ifdef MAXOCTETS
 static int
 setmodir(argv)
@@ -1685,6 +1687,121 @@ loadplugin(argv)
     return 1;
 }
 
+
+/*
+ * options_from_file - Read a string of options from controller file descriptor,
+ * and interpret them.
+ */
+int
+options_from_controller()
+{
+    int i, newline, ret;
+    option_t *opt;
+    int n, oldpriv;
+    char *argv[MAXARGS];
+    char args[MAXARGS][MAXWORDLEN];
+    char cmd[MAXWORDLEN];
+
+    oldpriv = privileged_option;
+    privileged_option = (controlfd_uid == 0);
+    option_source = "controller";
+    option_priority = OPRIO_CMDLINE;
+    ret = 0;
+
+    while (getword(controlfile, cmd, &newline, "controller")) {
+    
+        if (!strcmp(cmd, "[OPTIONS]"))
+            continue;
+        if (!strcmp(cmd, "[EOP]"))
+            break;
+
+	opt = find_option(cmd);
+	if (opt == NULL) {
+	    option_error("In controller file descriptor: unrecognized option '%s'",
+			 cmd);
+	    goto err;
+	}
+	n = n_arguments(opt);
+	for (i = 0; i < n; ++i) {
+	    if (!getword(controlfile, args[i], &newline, "controller")) {
+		option_error(
+			"In controller file descriptor: too few parameters for option '%s'",
+			cmd);
+		goto err;
+	    }
+	    argv[i] = args[i];
+	}
+	if (!process_option(opt, cmd, argv))
+	    goto err;
+    }
+    ret = 1;
+
+err:
+	privileged_option = oldpriv;
+    return ret;
+}
+
+/*
+ * controlled_connection - Prepare control and status file descriptors
+ */
+static int
+controlled_connection(argv)
+    char **argv;
+{
+    
+    int				len;
+	struct xucred   xucred;
+
+    /* first pipe STDIN */
+    
+
+    controlfd = dup(STDIN_FILENO);
+    if (controlfd == -1) {
+	option_error("Can't duplicate control file descripor: %m");
+	goto err;
+    }
+        
+    controlfile = fdopen(controlfd, "r");
+    if (controlfile == NULL) {
+        close(controlfd);
+	option_error("Can't open control file descripor: %m");
+	goto err;
+    }
+    
+	len = sizeof(xucred);
+	xucred.cr_uid = 7;
+	if (getsockopt(controlfd, 0, LOCAL_PEERCRED, &xucred, &len) == -1) {
+		option_error("Cannot get LOCAL_PEERCRED on control file descriptor (%m)\n");
+		goto err;
+	}
+
+    controlfd_uid = xucred.cr_uid;
+
+    /* then pipe STDOUT */
+
+    statusfd = dup(STDOUT_FILENO);
+    if (statusfd == -1) {
+	option_error("Can't duplicate status file descripor: %m");
+	goto err;
+    }
+
+    controlled = 1;
+    return 1;
+    
+err:
+    if (controlfile) {
+        fclose(controlfile); // also closes controlfd
+        controlfile = 0;
+        controlfd = -1;
+    }
+    
+    if (statusfd != -1) {
+        close(statusfd); 
+        statusfd = -1;
+    }
+    return 0;
+}
+
 #else
 static int
 loadplugin(argv)
